@@ -1,3 +1,4 @@
+require 'set'
 require 'rgen/ecore/ecore'
 
 module RGen
@@ -11,6 +12,7 @@ class ECoreToRuby
     @modules = {}
     @classifiers = {}
     @features_added = {}
+    @reserved = Set.new(Object.methods)
   end
 
   # Create a Ruby module representing +epackage+.
@@ -38,22 +40,35 @@ class ECoreToRuby
   # of a class/module with a temporary name is queried.
   #
   def create_module(epackage, under=Module.new)
-    temp = under.to_s.start_with?("#")
-    mod = create_module_internal(epackage, under, temp)
+    with_empty_constant_order_helper do
+      temp = under.to_s.start_with?("#")
+      mod = create_module_internal(epackage, under, temp)
 
-    epackage.eAllClassifiers.each do |c| 
-      if c.is_a?(RGen::ECore::EClass)
-        create_class(c, temp)
-      elsif c.is_a?(RGen::ECore::EEnum)
-        create_enum(c)
+      epackage.eAllClassifiers.each do |c| 
+        if c.is_a?(RGen::ECore::EClass)
+          create_class(c, temp)
+        elsif c.is_a?(RGen::ECore::EEnum)
+          create_enum(c)
+        end
       end
-    end
 
-    mod
+      load_classes_with_reserved_keywords(epackage)
+      mod
+    end
   end
 
   private
 
+  def load_classes_with_reserved_keywords(epackage)
+    epackage.eAllClassifiers.each do |eclass|
+      # we early load classes which have ruby reserved keywords
+      if eclass.is_a?(RGen::ECore::EClass)
+        reserved_used = eclass.eStructuralFeatures.any? { |f| @reserved.include?(f.name.to_sym) }
+        add_features(eclass) if reserved_used
+      end
+    end
+  end
+
   def create_module_internal(epackage, under, temp)
     return @modules[epackage] if @modules[epackage]
     
@@ -199,6 +214,27 @@ class ECoreToRuby
     end
   end
 
+  class EmptyConstantOrderHelper
+    def classCreated(c); end
+    def moduleCreated(m); end
+    def enumCreated(e); end
+  end
+
+  def with_empty_constant_order_helper
+    orig_coh = RGen::MetamodelBuilder::ConstantOrderHelper
+    RGen::MetamodelBuilder.instance_eval { remove_const(:ConstantOrderHelper) }
+    RGen::MetamodelBuilder.const_set(:ConstantOrderHelper, EmptyConstantOrderHelper.new)
+
+    begin
+      result = yield
+    ensure
+      RGen::MetamodelBuilder.instance_eval { remove_const(:ConstantOrderHelper) }
+      RGen::MetamodelBuilder.const_set(:ConstantOrderHelper, orig_coh)
+    end
+
+    result
+  end
+
   public
 
   def add_features(eclass)
