@@ -319,6 +319,10 @@ module BuilderExtensions
         end
         <% if name != "class" %>
           alias <%= name %> get<%= firstToUpper(name) %>
+        <% else %>
+          def getGeneric(role)
+            send("get\#{firstToUpper(role.to_s)}")
+          end
         <% end %>
 
       CODE
@@ -380,10 +384,14 @@ module BuilderExtensions
       @@many_read_builder ||= ERB.new <<-CODE
       
         def get<%= firstToUpper(name) %>
-          ( @<%= name %> ? @<%= name %>.dup : [] )
+          ( defined?(@<%= name %>) ? @<%= name %>.dup : [] )
         end
         <% if name != "class" %>
           alias <%= name %> get<%= firstToUpper(name) %>
+        <% else %>
+          def getGeneric(role)
+            send("get\#{firstToUpper(role.to_s)}")
+          end
         <% end %>
               
       CODE
@@ -394,8 +402,8 @@ module BuilderExtensions
       @@many_write_builder ||= ERB.new <<-CODE
     
         def add<%= firstToUpper(name) %>(val, index=-1)
-          @<%= name %> = [] unless @<%= name %>
-          return if val.nil? || (@<%= name %>.any?{|e| e.object_id == val.object_id} && (val.is_a?(MMBase) || val.is_a?(MMGeneric)))
+          @<%= name %> = [] unless defined?(@<%= name %>)
+          return if val.nil? || (val.is_a?(MMBase) || val.is_a?(MMGeneric)) && @<%= name %>.any? {|e| e.equal?(val)}
           <%= type_check_code("val", props) %>
           @<%= name %>.insert(index, val)
           <% if other_role %>
@@ -407,9 +415,9 @@ module BuilderExtensions
         end
         
         def remove<%= firstToUpper(name) %>(val)
-          @<%= name %> = [] unless @<%= name %>
+          @<%= name %> = [] unless defined?(@<%= name %>)
           @<%= name %>.each_with_index do |e,i|
-            if e.object_id == val.object_id
+            if e.equal?(val)
               @<%= name %>.delete_at(i)
               <% if props.reference? && props.value(:containment) %>
                 val._set_container(nil, nil)
@@ -428,14 +436,30 @@ module BuilderExtensions
           get<%= firstToUpper(name) %>.each {|e|
             remove<%= firstToUpper(name) %>(e)
           }
-          val.each {|v|
-            add<%= firstToUpper(name) %>(v)
+          @<%= name %> = [] unless defined?(@<%= name %>)
+          <% if props.reference? %>
+          val.uniq {|elem| elem.object_id }.each {|elem|
+            next if elem.nil?
+            <%= type_check_code("elem", props) %>
+            @<%= name %> << elem
+            <% if other_role %>
+              elem._register<%= firstToUpper(other_role) %>(self) unless elem.is_a?(MMGeneric)
+            <% end %>
+            <% if props.value(:containment) %>
+              elem._set_container(self, :<%= name %>)
+            <% end %>
           }
+          <% else %>
+          val.each {|elem|
+            <%= type_check_code("elem", props) %>
+            @<%= name %> << elem
+         }
+         <% end %>
         end
         alias <%= name %>= set<%= firstToUpper(name) %>
         
         def _register<%= firstToUpper(name) %>(val)
-          @<%= name %> = [] unless @<%= name %>
+          @<%= name %> = [] unless defined?(@<%= name %>)
           @<%= name %>.push val
           <% if props.reference? && props.value(:containment) %>
             val._set_container(self, :<%= name %>)
@@ -487,7 +511,7 @@ module BuilderExtensions
 
   def check_default_value_literal(literal, props)
     return if literal.nil? || props.impl_type == String
-    if props.impl_type == Integer
+    if props.impl_type == Integer || props.impl_type == RGen::MetamodelBuilder::DataTypes::Long
       unless literal =~ /^\d+$/
         raise StandardError.new("Property #{props.value(:name)} can not take value #{literal}, expected an Integer")
       end
@@ -510,8 +534,12 @@ module BuilderExtensions
   
   def type_check_code(varname, props)
     code = ""
-    if props.impl_type.is_a?(Class)
-      code << "unless #{varname}.nil? || #{varname}.is_a?(#{props.impl_type}) || #{varname}.is_a?(MMGeneric)"
+    if props.impl_type == RGen::MetamodelBuilder::DataTypes::Long
+      code << "unless #{varname}.nil? || #{varname}.is_a?(Integer) || #{varname}.is_a?(MMGeneric)"
+      code << "\n"
+      expected = "Integer"
+    elsif props.impl_type.is_a?(Class)
+      code << "unless #{varname}.nil? || #{varname}.is_a?(ObjectSpace._id2ref(#{props.impl_type.object_id})) || #{varname}.is_a?(MMGeneric)"
       code << " || #{varname}.is_a?(BigDecimal)" if props.impl_type == Float && defined?(BigDecimal)
       code << "\n"
       expected = props.impl_type.to_s
